(* Delphi Dialog
   Besttigungsdialog mit CheckBox
   ===============================
   
    J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de))

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
   
   Vers. 1 - Feb. 2014
   last modified: Nov. 2021
   *)

unit ConfirmCheckDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Vcl.Imaging.pngimage;

type
  TConfirmCheckDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    cbConfirm: TCheckBox;
    imgQMark: TImage;
    laMsg: TStaticText;
    laDir: TStaticText;
    laCaption: TStaticText;
    procedure FormCreate(Sender: TObject);
    procedure cbConfirmClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    FDir : string;
  public
    { Public declarations }
{$IFDEF HDPI}   // scale glyphs and images for High DPI
    procedure AfterConstruction; override;
{$EndIf}
    function Execute (APos : TPoint; const ATitle,ACaption,Desc,ADir,Confirm : string) : TModalresult;
  end;

function ConfirmCheck (APos : TPoint; const ACaption,Desc,ADir,Confirm : string) : TModalresult;

var
  ConfirmCheckDialog: TConfirmCheckDialog;

{ ---------------------------------------------------------------- }
implementation

{$R *.DFM}

uses Vcl.FileCtrl, WinUtils, GnuGetText;

{ ---------------------------------------------------------------- }
procedure TConfirmCheckDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

{$IFDEF HDPI}   // scale glyphs and images for High DPI
procedure TConfirmCheckDialog.AfterConstruction;
begin
  inherited;
  if Application.Tag=0 then
    ScaleButtonGlyphs(self,PixelsPerInchOnDesign,Monitor.PixelsPerInch);
  end;
{$EndIf}

procedure TConfirmCheckDialog.FormResize(Sender: TObject);
begin
  with laDir do Caption:=MinimizeName(FDir,Canvas,Width);
  end;

{ ---------------------------------------------------------------- }
procedure TConfirmCheckDialog.cbConfirmClick(Sender: TObject);
begin
  with OKBtn do if cbConfirm.Checked then begin
    ModalResult:=mrYes;
    Caption:=dgettext('dialogs','Yes');
    end
  else begin
    ModalResult:=mrNo;
    Caption:=dgettext('dialogs','No');
    end;
  end;

function TConfirmCheckDialog.Execute (APos : TPoint; const ATitle,ACaption,Desc,ADir,Confirm : string) : TModalresult;
begin
  AdjustFormPosition(Screen,self,APos);
  if length(ATitle)>0 then Caption:=ATitle;
  laMsg.Caption:=Desc;
  FDir:=ADir;
  laCaption.Caption:=ACaption;
  with laDir do Caption:=MinimizeName(ADir,Canvas,Width);
  with cbConfirm do begin
    Caption:=Confirm;
    Checked:=false;
    end;
  Result:=ShowModal;
  end;

function ConfirmCheck(APos : TPoint; const ACaption,Desc,ADir,Confirm : string) : TModalresult;
begin
  if not assigned(ConfirmCheckDialog) then
    ConfirmCheckDialog:=TConfirmCheckDialog.Create(Application);
  Result:=ConfirmCheckDialog.Execute(APos,'',ACaption,Desc,ADir,Confirm);
  FreeAndNil(ConfirmCheckDialog);
  end;

end.
