(* Delphi Dialog
   Texteingabe mit Maskenvorgabe
   =============================

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Jul. 2008
   last modified: Nov. 2021
   *)

unit InpMaskedText;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Vcl.Mask;

type
  TInputMaskedTextDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Descriptor: TLabel;
    TextFeld: TMaskEdit;
    procedure FormCreate(Sender: TObject);
    procedure TextFeldKeyPress(Sender: TObject; var Key: Char);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Private declarations }
    FIniName,FIniSection : string;
{$IFDEF HDPI}   // scale glyphs and images for High DPI
    procedure AfterConstruction; override;
{$EndIf}
  public
    { Public declarations }
  function Execute (APos       : TPoint;
                    const Titel,Desc,
                    Mask       : string;
                    var AText  : string) : boolean;
  end;

(* Text eingeben, Ergebnis: "true" bei "ok" *)
function InputMaskedText(APos       : TPoint;
                         const Titel,Desc,
                         Mask       : string;
                         var AText  : string) : boolean;

var
  InputMaskedTextDialog: TInputMaskedTextDialog;

implementation

{$R *.DFM}

uses CharTableDlg, System.IniFiles, WinUtils, GnuGetText;

procedure TInputMaskedTextDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if Modalresult=mrCancel then TextFeld.EditMask:='';
  end;

procedure TInputMaskedTextDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

{$IFDEF HDPI}   // scale glyphs and images for High DPI
procedure TInputMaskedTextDialog.AfterConstruction;
begin
  inherited;
  if Application.Tag=0 then
    ScaleButtonGlyphs(self,PixelsPerInchOnDesign,Monitor.PixelsPerInch);
  end;
{$EndIf}

{ ------------------------------------------------------------------- }
procedure TInputMaskedTextDialog.TextFeldKeyPress(Sender: TObject;
  var Key: Char);
begin
  if Key=#$1B then Modalresult:=mrCancel
  else if Key=#$D then Modalresult:=mrOK;
  end;

{ ------------------------------------------------------------------- }
function TInputMaskedTextDialog.Execute (APos : TPoint;
                                   const Titel,Desc,Mask : string;
                                   var AText  : string) : boolean;
begin
  AdjustFormPosition(Screen,self,APos);
  Caption:=Titel;
  Descriptor.Caption:=Desc;
  ActiveControl:=TextFeld;
  with TextFeld do begin
    Text:=AText;
    EditMask:=Mask;
    end;
  if ShowModal=mrOK then begin
    AText:=TextFeld.Text;
    Result:=true;
    end
  else Result:=false;
  end;

{ ------------------------------------------------------------------- }
(* Txt eingeben, Ergebnis: "true" bei "ok" *)
function InputMaskedText(APos       : TPoint;
                         const Titel,Desc,Mask : string;
                         var AText  : string) : boolean;
begin
  if not assigned(InputMaskedTextDialog) then
    InputMaskedTextDialog:=TInputMaskedTextDialog.Create(Application);
  with InputMaskedTextDialog do begin
    Result:=Execute(APos,Titel,Desc,Mask,AText);
    Release;
    end;
  InputMaskedTextDialog:=nil;
  end;

end.
