(*  Delphi Dialog
    Seiteneinrichtung fr Syn-Edit
    ==============================

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Jun. 2005
   last modified: Nov. 2021
   
   Requires routines from the SynEditPackage
   *)

unit SynEdPageDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, SynEditPrint,
  Vcl.Dialogs, Vcl.ComCtrls;

type
  TPrintSettings = record
    Ori         : TPrinterOrientation;
    HtHeader    : integer;
    end;

  TPageFormatDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    GroupBox1: TGroupBox;
    PageControl: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    cbxLineNumbers: TCheckBox;
    cbxWrapLines: TCheckBox;
    Label11: TLabel;
    cbxDuplex: TCheckBox;
    Label12: TLabel;
    cbxHighlight: TCheckBox;
    cbxColorPrint: TCheckBox;
    Label13: TLabel;
    edtFont: TEdit;
    FontDialog: TFontDialog;
    btnFont: TSpeedButton;
    Label14: TLabel;
    edtSize: TEdit;
    Label15: TLabel;
    edtStyle: TEdit;
    Label16: TLabel;
    GroupBox2: TGroupBox;
    rgpOrientation: TRadioGroup;
    cbxHeader: TCheckBox;
    pnlHeader: TPanel;
    edtLeft: TEdit;
    udLeft: TUpDown;
    edtRight: TEdit;
    udRight: TUpDown;
    edtGutter: TEdit;
    udGutter: TUpDown;
    edtTop: TEdit;
    udTop: TUpDown;
    edtBottom: TEdit;
    udBottom: TUpDown;
    edtHeader: TEdit;
    udHeader: TUpDown;
    procedure btnFontClick(Sender: TObject);
    procedure cbxHeaderClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    FFont : TFont;
    FontStyles : array [0..3] of string;
    procedure ShowFont;
  public
    { Public declarations }
    procedure LoadFromIni (const AIniName,ASection : string;
                           ADefRand                : TRect;
                           var APrintSettings      : TPrintSettings;
                           var ASynEditPrint       : TSynEditPrint);

    procedure SaveToIni (const AIniName,ASection : string;
                         var APrintSettings      : TPrintSettings;
                         var ASynEditPrint       : TSynEditPrint);

    function Execute (const ATitle      : string;
                      Listing           : boolean;
                      var PrtSettings   : TPrintSettings;
                      var ASynEditPrint : TSynEditPrint) : boolean;
  end;

var
  PageFormatDialog: TPageFormatDialog;

implementation

{$R *.DFM}

uses WinUtils, System.IniFiles, Vcl.Printers, GnuGetText;

const
  IniOrientation = 'Orientation';
  IniHeadLine = 'HeadLine';
  IniColors = 'Color';
  IniHighL  = 'Highlight';
  IniLnNum = 'LineNumbers';
  IniWrap   = 'Wrap';
  IniLeftMarg = 'LeftMargin';
  IniRightMarg = 'RightMargin';
  IniTopMarg = 'TopMargin';
  IniBottomMarg = 'BottomMargin';
  IniGutter = 'Gutter';
  IniMirror = 'Mirror';

{------------------------------------------------------------------- }
procedure TPageFormatDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  FontStyles[0]:=dgettext('dialogs','Normal');
  FontStyles[1]:=dgettext('dialogs','Bold');
  FontStyles[2]:=dgettext('dialogs','Italic');
  FontStyles[3]:=dgettext('dialogs','Bold italic');
  end;

procedure TPageFormatDialog.LoadFromIni (const AIniName,ASection : string;
                                         ADefRand                : TRect;
                                         var APrintSettings      : TPrintSettings;
                                         var ASynEditPrint       : TSynEditPrint);
var
  n : integer;
begin
  with TMemIniFile.Create(AIniName) do begin
    with APrintSettings do begin
      n:=ReadInteger (ASection,IniOrientation,ord(poLandscape));
      if (n<0) or (n>1) then n:=1;
      Ori:=TPrinterOrientation(n);
      HtHeader:=ReadInteger(ASection,IniHeadLine,5);
      end;
    with ASynEditPrint do begin
      Colors:=ReadBool(ASection,IniColors,false);
      Highlight:=ReadBool(ASection,IniHighL,false);
      LineNumbers:=ReadBool(ASection,IniLnNum,false);
      Wrap:=ReadBool(ASection,IniWrap,true);
      with Margins do begin
        Left:=ReadInteger(ASection,IniLeftMarg,ADefRand.Left);
        Right:=ReadInteger(ASection,IniRightMarg,ADefRand.Right);
        Top:=ReadInteger(ASection,IniTopMarg,ADefRand.Top);
        Bottom:=ReadInteger(ASection,IniBottomMarg,ADefRand.Bottom);
        Gutter:=ReadInteger(ASection,IniGutter,0);
        MirrorMargins:=ReadBool(ASection,IniMirror,false);
        end;
      end;
    Free;
    end;
  end;

procedure TPageFormatDialog.SaveToIni (const AIniName,ASection : string;
                                       var APrintSettings      : TPrintSettings;
                                       var ASynEditPrint       : TSynEditPrint);
begin
  with TMemIniFile.Create(AIniName) do begin
    with APrintSettings do begin
      WriteInteger (ASection,IniOrientation,ord(Ori));
      WriteInteger(ASection,IniHeadLine,HtHeader);
      end;
    with aSynEditPrint do begin
      WriteBool(ASection,IniColors,Colors);
      WriteBool(ASection,IniHighL,Highlight);
      WriteBool(ASection,IniLnNum,LineNumbers);
      WriteBool(ASection,IniWrap,Wrap);
      with Margins do begin
        WriteInteger(ASection,IniLeftMarg,round(Left));
        WriteInteger(ASection,IniRightMarg,round(Right));
        WriteInteger(ASection,IniTopMarg,round(Top));
        WriteInteger(ASection,IniBottomMarg,round(Bottom));
        WriteInteger(ASection,IniGutter,round(Gutter));
        WriteBool(ASection,IniMirror,MirrorMargins);
        end;
      end;
    UpdateFile;
    Free;
    end;
  end;

procedure TPageFormatDialog.ShowFont;
var
  fc      : TFontStyleToByte;
begin
  with FFont do begin
    edtFont.Text:=Name;
    edtSize.Text:=IntToStr(abs(Size));
    fc.Style:=Style;
    edtStyle.Text:=FontStyles[fc.Value and 3];
    end;
  end;

procedure TPageFormatDialog.btnFontClick(Sender: TObject);
begin
  with FontDialog do begin
    Font:=FFont;
    if Execute then begin
      FFont:=Font;
      ShowFont;
      end;
    end;
  end;

procedure TPageFormatDialog.cbxHeaderClick(Sender: TObject);
begin
  pnlHeader.Visible:=cbxHeader.Checked;
  end;

{------------------------------------------------------------------- }
function TPageFormatDialog.Execute (const ATitle      : string;
                                    Listing           : boolean;
                                    var PrtSettings   : TPrintSettings;
                                    var ASynEditPrint : TSynEditPrint) : boolean;
begin
  Caption:=ATitle;
  with PageControl do if Listing then ActivePageIndex:=1 else ActivePageIndex:=0;
  with ASynEditPrint do begin
    with Margins do begin
      udLeft.Position:=round(Left);
      udRight.Position:=round(Right);
      udTop.Position:=round(Top);
      udBottom.Position:=round(Bottom);
      udGutter.Position:=round(Gutter);
      cbxDuplex.Checked:=MirrorMargins;
      end;
    with PrtSettings do begin
      rgpOrientation.ItemIndex:=integer(Ori);
      cbxHeader.Checked:=HtHeader>0;
      udHeader.Position:=HtHeader;
      pnlHeader.Visible:=cbxHeader.Checked;
      end;
    cbxLineNumbers.Checked:=LineNumbers;
    cbxWrapLines.Checked:=Wrap;
    cbxHighlight.Checked:=Highlight;
    cbxColorPrint.Checked:=Colors;
    FFont:=Font;
    ShowFont;
    if ShowModal=mrOK then begin
      with Margins do begin
        Left:=udLeft.Position;
        Right:=udRight.Position;
        Top:=udTop.Position;
        Bottom:=udBottom.Position;
        Gutter:=udGutter.Position;
        MirrorMargins:=cbxDuplex.Checked;
        end;
      with PrtSettings do begin
        Ori:=TPrinterOrientation(rgpOrientation.ItemIndex);
        if cbxHeader.Checked then HtHeader:=udHeader.Position
        else HtHeader:=0;
        end;
      LineNumbers:=cbxLineNumbers.Checked;
      Wrap:=cbxWrapLines.Checked;
      Highlight:=cbxHighlight.Checked;
      Colors:=cbxColorPrint.Checked;
      Font:=FFont;
      Result:=true;
      end
    else Result:=false;
    end;
  end;

end.
