(* Delphi Dialog
   Importieren von Text (z.B. csv-Format)

    J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de))

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   letzte nderung: Aug. 2008
   last modified: Nov. 2021
   *)

unit CsvImportDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, NumberEd;

const
  lmax = 9;

type
  TCsvImportDialog = class(TForm)
    Panel1: TPanel;
    Label3: TLabel;
    mePreview: TMemo;
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    edFilename: TEdit;
    gbDelim: TGroupBox;
    rbSpace: TRadioButton;
    rbTab: TRadioButton;
    rbSemi: TRadioButton;
    rbComma: TRadioButton;
    rbOtherDel: TRadioButton;
    edDelim: TEdit;
    rbSlash: TRadioButton;
    gbQuote: TGroupBox;
    rbNoQuote: TRadioButton;
    rbQuote: TRadioButton;
    rbOtherQuote: TRadioButton;
    edQuote: TEdit;
    gbLine: TGroupBox;
    Label4: TLabel;
    rgDecimal: TRadioGroup;
    GroupBox1: TGroupBox;
    edCol1: TLabeledEdit;
    edCol2: TLabeledEdit;
    edCol3: TLabeledEdit;
    edCol4: TLabeledEdit;
    reLine: TRangeEdit;
    udLine: TNumUpDown;
    procedure FormCreate(Sender: TObject);
    procedure rbTabClick(Sender: TObject);
    procedure rbSemiClick(Sender: TObject);
    procedure rbSlashClick(Sender: TObject);
    procedure rbSpaceClick(Sender: TObject);
    procedure rbCommaClick(Sender: TObject);
    procedure rbOtherDelClick(Sender: TObject);
    procedure rbNoQuoteClick(Sender: TObject);
    procedure rbQuoteClick(Sender: TObject);
    procedure rbOtherQuoteClick(Sender: TObject);
    procedure rgDecimalClick(Sender: TObject);
    procedure reLineChange(Sender: TObject);
  private
    { Private declarations }
    NDel,NQuote,DecSep : char;
{$IFDEF HDPI}   // scale glyphs and images for High DPI
    procedure AfterConstruction; override;
{$EndIf}
    procedure UpdatePreview;
  public
    { Public declarations }
    function Execute (AFilename : string;
                      var Delim,Quote,Dec : char;
                      var FLine : integer) : boolean;
  end;

var
  CsvImportDialog: TCsvImportDialog;

implementation

{$R *.dfm}

uses StringUtils, FileUtils, GgtDummy;

procedure TCsvImportDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

{$IFDEF HDPI}   // scale glyphs and images for High DPI
procedure TCsvImportDialog.AfterConstruction;
begin
  inherited;
  if Application.Tag=0 then
    ScaleButtonGlyphs(self,PixelsPerInchOnDesign,Monitor.PixelsPerInch);
  end;
{$EndIf}

procedure TCsvImportDialog.UpdatePreview;
var
  n,k : integer;
  s,t : string;
begin
//  if not Visible then Exit;
  n:=reLine.Value;
  with mePreView.Lines do if n>Count then n:=Count;
  s:=mePreview.Lines[n-1];
  edCol1.Text:=''; edCol2.Text:='';
  edCol3.Text:=''; edCol4.Text:='';
  k:=0;
  repeat
    if NQuote=#0 then t:=ReadNxtStr(s,NDel)
    else t:=ReadNxtQuotedStr(s,NDel,NQuote);
    inc(k);
    if length(t)>0 then begin
      case k of
      1 : edCol1.Text:=t;
      2 : edCol2.Text:=t;
      3 : edCol3.Text:=t;
      4 : edCol4.Text:=t;
        end;
      end;
    until length(s)=0;
  end;

procedure TCsvImportDialog.rbTabClick(Sender: TObject);
begin
  NDel:=Tab;
  edDelim.Hide;
  UpdatePreview;
  end;

procedure TCsvImportDialog.rbSpaceClick(Sender: TObject);
begin
  NDel:=Space;
  edDelim.Hide;
  UpdatePreview;
  end;

procedure TCsvImportDialog.rbSemiClick(Sender: TObject);
begin
  NDel:=';';
  edDelim.Hide;
  UpdatePreview;
  end;

procedure TCsvImportDialog.rbCommaClick(Sender: TObject);
begin
  NDel:=',';
  edDelim.Hide;
  UpdatePreview;
  end;

procedure TCsvImportDialog.rbSlashClick(Sender: TObject);
begin
  NDel:='/';
  edDelim.Hide;
  UpdatePreview;
  end;

procedure TCsvImportDialog.rbOtherDelClick(Sender: TObject);
begin
  with edDelim do begin
    if length(Text)>0 then NDel:=Text[1] else NDel:=',';
    Text:=NDel;
    Show;
    end;
  UpdatePreview;
  end;

procedure TCsvImportDialog.rbNoQuoteClick(Sender: TObject);
begin
  NQuote:=#0;
  edQuote.Hide;
  UpdatePreview;
  end;

procedure TCsvImportDialog.rbQuoteClick(Sender: TObject);
begin
  NQuote:='"';
  edQuote.Hide;
  UpdatePreview;
  end;

procedure TCsvImportDialog.rbOtherQuoteClick(Sender: TObject);
begin
  with edQuote do begin
    if length(Text)>0 then NQuote:=Text[1] else NQuote:='''';
    Text:=NQuote;
    Show;
    end;
  UpdatePreview;
  end;

procedure TCsvImportDialog.rgDecimalClick(Sender: TObject);
begin
  if rgDecimal.ItemIndex=0 then DecSep:=Period else DecSep:=',';
  UpdatePreview;
  end;

procedure TCsvImportDialog.reLineChange(Sender: TObject);
begin
  UpdatePreview;
  end;

function TCsvImportDialog.Execute (AFilename : string;
                                var Delim,Quote,Dec : char;
                                var FLine : integer) : boolean;
var
  fi      : TextFile;
  s       : string;
  i       : integer;
begin
  edFilename.Text:=StripPath(AFilename,35);
  NDel:=Delim; NQuote:=Quote;
  edDelim.Text:=''; edQuote.Text:='';
  gbLine.Visible:=FLine>0;
  reLine.Value:=FLine;
  AssignFile(fi,AFilename); reset(fi);
  with mePreview do begin
    Clear;
    i:=1;
    repeat
      readln(fi,s);
      Lines.Add(s);
      inc(i);
      until (i>lmax) or Eof(fi);
    CloseFile(fi);
    end;
  edDelim.Hide;
  case NDel of
  Space     : rbSpace.Checked:=true;
  Tab       : rbTab.Checked:=true;
  ';'       : rbSemi.Checked:=true;
  ','       : rbComma.Checked:=true;
  '/'       : rbSlash.Checked:=true;
  else with edDelim do begin
      rbOtherDel.Checked:=true;
      Text:=NDel; Show;
      end;
    end;
  DecSep:=Dec;
  with rgDecimal do begin
    Visible:=Dec<>#0;
    case Dec of
      Period : ItemIndex:=0;
      ','    : ItemIndex:=1;
      end;
    end;
  edQuote.Hide;
  case NQuote of
  '"' : rbQuote.Checked:=true;
  #0  : rbNoQuote.Checked:=true;
  else with edQuote do begin
      rbOtherQuote.Checked:=true;
      Text:=NQuote; Show;
      end;
    end;
  UpdatePreview;
  if ShowModal=mrOK then begin
    Delim:=NDel;
    Quote:=NQuote;
    if rgDecimal.Visible then Dec:=DecSep;
    if gbLine.Visible then FLine:=reLine.Value;
    Result:=true;
    end
  else Result:=false;
  end;

end.
