(* Delphi Dialog
   Lschen und Bearbeiten von Eintrgen in Listen (TStrings)
   =========================================================
   z.B. geeignet zur Wartung von History-Listen

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - Apr. 2005
   Vers. 2 - Feb. 2020: Mehrfach-Auswahl fr Verschieben und Lschen
   last modified: Nov. 2021
    *)

unit EditHistListDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls;

type
  TEditHistListDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    lbxStringList: TListBox;
    lbDesc: TLabel;
    btnDelete: TBitBtn;
    gbxMove: TGroupBox;
    UpBtn: TBitBtn;
    DownBtn: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure UpBtnClick(Sender: TObject);
    procedure DownBtnClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
{$IFDEF HDPI}   // scale glyphs and images for High DPI
    procedure AfterConstruction; override;
{$EndIf}
    function Execute (APos : TPoint; Titel,Desc : string;
                      HList : TStrings; ADelete : boolean;
                      var AIndex : integer) : boolean;
  end;

function EditHistList(APos : TPoint; Titel,Desc : string;
                      HList : TStrings; ADelete : boolean;
                      var AIndex : integer) : boolean; overload;
function EditHistList(APos : TPoint; Titel,Desc : string;
                      HList : TStrings) : boolean; overload;

var
  EditHistListDialog: TEditHistListDialog;

implementation

{$R *.DFM}

uses InpText, WinUtils, GgtDummy;

{------------------------------------------------------------------- }
procedure TEditHistListDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

{$IFDEF HDPI}   // scale glyphs and images for High DPI
procedure TEditHistListDialog.AfterConstruction;
begin
  inherited;
  ScaleButtonGlyphs(self,PixelsPerInchOnDesign,Monitor.PixelsPerInch);
  end;
{$EndIf}

{------------------------------------------------------------------- }
procedure TEditHistListDialog.btnDeleteClick(Sender: TObject);
var
  i,n : integer;
begin
  with lbxStringList do if (SelCount>0) then begin
    n:=ItemIndex;
    for i:=Count-1 downto 0 do if Selected[i] then Items.Delete(i);
    if n>Items.Count then ItemIndex:=Items.Count-1 else ItemIndex:=n;
    end;
  end;

procedure TEditHistListDialog.UpBtnClick(Sender: TObject);
var
  i,n,k : integer;
  IndexList : array of integer;
begin
  with lbxStringList do if (Count>0) then begin
    SetLength(IndexList,Count);
    n:=0;
    for i:=0 to Count-1 do if Selected[i] then begin
      IndexList[n]:=i; inc(n);
      end;
    SetLength(IndexList,n);  // list of selected items
    ClearSelection; k:=0;
    for i:=0 to High(IndexList) do begin // move selected items
      if IndexList[i]>k then begin
        n:=IndexList[i]-1;
        Items.Exchange(IndexList[i],n);
        IndexList[i]:=n;
        end
      else n:=IndexList[i];
      if k=n then inc(k);
      end;
    for i:=0 to High(IndexList) do Selected[IndexList[i]]:=true; //update selection
    ItemIndex:=IndexList[0];
    end;
  end;

procedure TEditHistListDialog.DownBtnClick(Sender: TObject);
var
  i,n,k : integer;
  IndexList : array of integer;
begin
  with lbxStringList do if (Count>0) then begin
    SetLength(IndexList,Count);
    n:=0;
    for i:=Count-1 downto 0 do if Selected[i] then begin
      IndexList[n]:=i; inc(n);
      end;
    SetLength(IndexList,n);  // list of selected items
    ClearSelection; k:=Count-1;
    for i:=0 to High(IndexList) do begin // move selected items
      if IndexList[i]<k then begin
        n:=IndexList[i]+1;
        Items.Exchange(IndexList[i],n);
        IndexList[i]:=n;
        end
      else n:=IndexList[i];
      if k=n then dec(k);
      end;
    for i:=0 to High(IndexList) do Selected[IndexList[i]]:=true; //update selection
    ItemIndex:=IndexList[0];
    end;
  end;

{------------------------------------------------------------------- }
function TEditHistListDialog.Execute (APos : TPoint; Titel,Desc : string;
                    HList : TStrings; ADelete : boolean;
                    var AIndex : integer) : boolean;
var
  i : integer;
  s : string;
begin
  AdjustFormPosition(Screen,self,APos);
  if length(Titel)>0 then Caption:=Titel;
  lbDesc.Caption:=Desc;
  lbxStringList.Clear;
  btnDelete.Visible:=ADelete;
  with HList do for i:=0 to Count-1 do lbxStringList.AddItem(Strings[i],nil);
  if (AIndex>=0) and (AIndex<HList.Count) then begin
    s:=HList.Strings[AIndex];
    lbxStringList.ItemIndex:=AIndex;
    end
  else s:='';
  if ShowModal=mrOK then begin
    HList.Clear;
    with lbxStringList.Items do begin
      for i:=0 to Count-1 do
        HList.Add(Strings[i]);
      if length(s)>0 then begin
        i:=IndexOf(s);
        if i>=0 then AIndex:=i else AIndex:=-1;
        end;
      end;
    Result:=true;
    end
  else Result:=false;
  end;

function EditHistList(APos : TPoint; Titel,Desc : string;
                      HList : TStrings; ADelete : boolean;
                      var AIndex : integer) : boolean;
begin
  if not assigned(EditHistListDialog) then begin
    EditHistListDialog:=TEditHistListDialog.Create(Application);
    end;
  Result:=EditHistListDialog.Execute(APos,Titel,Desc,HList,ADelete,AIndex);
  FreeAndNil(EditHistListDialog);
  end;

function EditHistList(APos : TPoint; Titel,Desc : string;
                      HList : TStrings) : boolean;
var
  n : integer;
begin
  Result:=EditHistList(APos,Titel,Desc,Hlist,true,n);
  end;

end.
