(* Delphi Dialog
   String-Liste anzeigen
   =====================
   
    J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de))

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
   
   Vers. 1 - Dez. 2021
   *)

unit ShowStringList;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls;

type
  TShowStringListDialog = class(TForm)
    btnClose: TBitBtn;
    laDesc: TLabel;
    lbStrings: TListBox;
    btnCopy: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure btnCopyClick(Sender: TObject);
  private
    { Private declarations }
{$IFDEF HDPI}   // scale glyphs and images for High DPI
    procedure AfterConstruction; override;
{$EndIf}
  public
    { Public declarations }
    procedure Execute (APos : TPoint; const Titel,Desc : string; StrList : TStringList); overload;
    procedure Execute (const Titel,Desc : string; StrList : TStringList); overload;
  end;

var
  ShowStringListDialog: TShowStringListDialog;

{ ---------------------------------------------------------------- }
implementation

{$R *.DFM}

uses GgtDummy, WinUtils, Vcl.ClipBrd;

{ ---------------------------------------------------------------- }
procedure TShowStringListDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

{$IFDEF HDPI}   // scale glyphs and images for High DPI
procedure TShowStringListDialog.AfterConstruction;
begin
  inherited;
  if Application.Tag=0 then begin
    ScaleButtonGlyphs(self,PixelsPerInchOnDesign,Monitor.PixelsPerInch);
    end;
  end;
{$EndIf}

procedure TShowStringListDialog.btnCopyClick(Sender: TObject);
begin
  ClipBoard.AsText:=Caption+sLineBreak+laDesc.Caption+sLineBreak+lbStrings.Items.Text;
  end;

{ ---------------------------------------------------------------- }
procedure TShowStringListDialog.Execute (APos : TPoint; const Titel,Desc : string; StrList : TStringList);
begin
  AdjustFormPosition(Screen,self,APos);
  Caption:=Titel;
  laDesc.Caption:=Desc;
  lbStrings.Items.Assign(StrList);
  ShowModal;
  end;

procedure TShowStringListDialog.Execute (const Titel,Desc : string; StrList : TStringList);
begin
  Execute (CenterPos,Titel,Desc,StrList);
  end;

end.
